from unittest.mock import patch

from badfish.helpers.exceptions import BadfishException
from tests.config import (
    BLANK_RESP,
    INIT_RESP,
    JOB_ID,
    JOB_OK_RESP,
    RESPONSE_CHECK_JOB,
    RESPONSE_CHECK_JOB_BAD,
    RESPONSE_CHECK_JOB_ERROR,
    RESPONSE_CLEAR_JOBS,
    RESPONSE_CLEAR_JOBS_LIST,
    RESPONSE_CLEAR_JOBS_LIST_EXCEPTION,
    RESPONSE_CLEAR_JOBS_UNSUPPORTED,
    RESPONSE_DELETE_JOBS_SUPPORTED_EXCEPTION,
    RESPONSE_DELETE_JOBS_UNSUPPORTED_EXCEPTION,
    RESPONSE_LS_JOBS,
    RESPONSE_LS_JOBS_EMPTY,
    TASK_OK_RESP,
)
from tests.test_base import TestBase


def raise_badfish_exception_stub_del_req(arg1=None, arg2=None):
    raise BadfishException


class TestJobQueue(TestBase):
    args = ["--ls-jobs"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_jobs(self, mock_get, mock_post, mock_delete):
        responses_add = [JOB_OK_RESP]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_LS_JOBS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_jobs_empty(self, mock_get, mock_post, mock_delete):
        responses_add = [BLANK_RESP]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_LS_JOBS_EMPTY


class TestClearJobs(TestBase):
    args = ["--clear-jobs"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_clear_jobs(self, mock_get, mock_post, mock_delete):
        responses_add = [
            JOB_OK_RESP,
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_CLEAR_JOBS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_clear_jobs_unsupported(self, mock_get, mock_post, mock_delete):
        responses_add = [
            JOB_OK_RESP,
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 400, 200], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_CLEAR_JOBS_UNSUPPORTED

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_clear_jobs_list(self, mock_get, mock_post, mock_delete):
        responses_add = [
            JOB_OK_RESP,
            BLANK_RESP,
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 400, 400, 400], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(
            mock_delete,
            [400, 400, 200, 200],
            ["Bad Request", "Bad Request", "OK", "OK"],
        )
        _, err = self.badfish_call()
        assert err == RESPONSE_CLEAR_JOBS_LIST

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_clear_jobs_force(self, mock_get, mock_post, mock_delete):
        force_arg = "--force"
        responses_add = [
            JOB_OK_RESP,
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = self.args + [force_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_CLEAR_JOBS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_clear_jobs_list_exception(self, mock_get, mock_post, mock_delete):
        responses_add = [
            JOB_OK_RESP,
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        status_codes = [200, 200, 200, 200, 200, 400, 200]
        self.set_mock_response(mock_get, status_codes, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 400, "Bad Request")
        _, err = self.badfish_call()
        assert err == RESPONSE_CLEAR_JOBS_LIST_EXCEPTION


class TestDeleteJob(TestBase):
    args = ["--clear-jobs"]

    @patch("aiohttp.ClientSession.delete", raise_badfish_exception_stub_del_req)
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_delete_unsupported_exception(self, mock_get, mock_post):
        responses_add = [JOB_OK_RESP]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 400, 200], responses)
        self.set_mock_response(mock_post, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_DELETE_JOBS_UNSUPPORTED_EXCEPTION

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_delete_supported_exception(self, mock_get, mock_post, mock_delete):
        responses_add = [JOB_OK_RESP]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["Bad Request", "OK"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_DELETE_JOBS_SUPPORTED_EXCEPTION


class TestCheckJob(TestBase):
    args = ["--check-job"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_job_ok(self, mock_get, mock_post, mock_delete):
        responses_add = [
            TASK_OK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = self.args + [JOB_ID]
        _, err = self.badfish_call()
        assert err == RESPONSE_CHECK_JOB

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_job_bad(self, mock_get, mock_post, mock_delete):
        responses_add = [
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 404], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = self.args + ["JID_WHICHDOESNOTEXIST"]
        _, err = self.badfish_call()
        assert err == RESPONSE_CHECK_JOB_BAD

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    @patch("badfish.main.Badfish.get_request")
    def test_check_job_error(self, mock_get_req_call, mock_get, mock_post, mock_delete):
        # The check_schedule_job_status method only makes one call to get_request
        # which should return None to simulate the error condition
        mock_get_req_call.side_effect = [None]
        self.set_mock_response(mock_get, 200, INIT_RESP)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = self.args + [JOB_ID]
        _, err = self.badfish_call()
        assert err == RESPONSE_CHECK_JOB_ERROR



from unittest.mock import patch

from tests.config import (
    FIRMWARE_INVENTORY_1_RESP,
    FIRMWARE_INVENTORY_2_RESP,
    FIRMWARE_INVENTORY_RESP,
    FIRMWARE_INVENTORY_RESP_CONTAINING_ERROR,
    INIT_RESP,
    RESPONSE_FIRMWARE_INVENTORY,
    RESPONSE_FIRMWARE_INVENTORY_NONE_RESPONSE,
    RESPONSE_FIRMWARE_INVENTORY_NOT_ABLE_TO_ACCESS,
)
from tests.test_base import MockResponse, TestBase


class TestFirmwareInventory(TestBase):
    option_arg = "--firmware-inventory"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_firmware_inventory(self, mock_get, mock_post, mock_delete):
        responses_add = [
            FIRMWARE_INVENTORY_RESP,
            FIRMWARE_INVENTORY_1_RESP,
            FIRMWARE_INVENTORY_2_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_FIRMWARE_INVENTORY

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_firmware_inventory_json_invalid(self, mock_get, mock_post, mock_delete):
        responses_add = [""]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_FIRMWARE_INVENTORY_NOT_ABLE_TO_ACCESS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_firmware_inventory_json_contains_error(self, mock_get, mock_post, mock_delete):
        responses_add = [FIRMWARE_INVENTORY_RESP_CONTAINING_ERROR]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_FIRMWARE_INVENTORY_NOT_ABLE_TO_ACCESS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    @patch("badfish.main.Badfish.get_request")
    def test_firmware_inventory_none_response(self, mock_get_req_call, mock_get, mock_post, mock_delete):
        mock_get_req_call.side_effect = [
            MockResponse(FIRMWARE_INVENTORY_RESP, 200),  # Firmware inventory list
            MockResponse(FIRMWARE_INVENTORY_1_RESP, 200),  # First device details
            None,  # Second device fails
        ]
        self.set_mock_response(mock_get, 200, INIT_RESP)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_FIRMWARE_INVENTORY_NONE_RESPONSE



import sys
from unittest.mock import AsyncMock, MagicMock, PropertyMock, patch

import pytest
from aiohttp import web
from aiohttp.test_utils import AioHTTPTestCase

from badfish.main import main
from badfish.helpers.exceptions import BadfishException
from tests import config


class TestBase(AioHTTPTestCase):
    patch("asyncio.sleep").start()
    patch("time.sleep").start()

    async def get_application(self):
        return web.Application()

    @staticmethod
    def set_mock_response(mock, status, responses, post=False, headers=None):
        mock.return_value.__aenter__.return_value.name = responses
        status_mock = MagicMock()
        if isinstance(status, list):
            if post:
                dup_stats = [val for val in status for _ in range(2)]
                type(status_mock).status = PropertyMock(side_effect=dup_stats)
            else:
                type(status_mock).status = PropertyMock(side_effect=status)
        else:
            type(status_mock).status = PropertyMock(return_value=status)
        mock.return_value.__aenter__.return_value = status_mock
        mock.return_value.__aenter__.return_value.read = AsyncMock()
        if isinstance(responses, list):
            mock.return_value.__aenter__.return_value.text = AsyncMock(side_effect=responses)
        else:
            mock.return_value.__aenter__.return_value.text = AsyncMock(return_value=responses)
        if headers is not None:
            mock.return_value.__aenter__.return_value.headers = headers

    @pytest.fixture(autouse=True)
    def inject_capsys(self, capsys):
        self._capsys = capsys

    @pytest.fixture(autouse=True)
    def capture_wrap(self):
        sys.stderr.close = lambda *args: None
        sys.stdout.close = lambda *args: None
        yield

    def badfish_call(
        self,
        mock_host=config.MOCK_HOST,
        mock_user=config.MOCK_USER,
        mock_pass=config.MOCK_PASS,
    ):
        argv = []

        if mock_host is not None:
            argv.extend(("-H", mock_host))
        if mock_user is not None:
            argv.extend(("-u", mock_user))
        if mock_pass is not None:
            argv.extend(("-p", mock_pass))

        argv.extend(self.args)
        try:
            main(argv)
        except BadfishException:
            pass
        out, err = self._capsys.readouterr()
        return out, err


class MockResponse:
    def __init__(self, text: str, status: int):
        self.txt = text
        self.status = status

    async def text(self, arg1, arg2):
        return self.txt

    async def __aexit__(self, exc_type, exc, tb):
        pass

    async def __aenter__(self):
        return self



from unittest.mock import patch

from tests.config import (
    BIOS_PASS_CHANGE_CMD_FAILED,
    BIOS_PASS_CHANGE_NOT_SUPPORTED,
    BIOS_PASS_RM_GOOD,
    BIOS_PASS_RM_MISS_ARG,
    BIOS_PASS_SET_CHECK_JOB_STATUS_BAD_CODE,
    BIOS_PASS_SET_CHECK_JOB_STATUS_FAIL_MSG,
    BIOS_PASS_SET_GOOD,
    BIOS_PASS_SET_MISS_ARG,
    BLANK_RESP,
    CHECK_JOB_STATUS_FAIL_MSG,
    CHECK_JOB_STATUS_UNEXPECTED_MSG_CONTENT,
    INIT_RESP,
    JOB_OK_RESP,
    RESET_TYPE_RESP,
    STATE_OFF_RESP,
    TASK_OK_RESP,
)
from tests.test_base import TestBase


class TestSetBiosPass(TestBase):
    option_arg = "--set-bios-password"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_set_bios_pass_ok(self, mock_get, mock_post, mock_delete):
        responses_get = [
            RESET_TYPE_RESP,
            STATE_OFF_RESP,
            TASK_OK_RESP,
        ]
        responses = INIT_RESP + responses_get
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_SET_GOOD

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_set_bios_pass_missing_arg(self, mock_get, mock_post, mock_delete):
        responses_get = [
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_get
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_SET_MISS_ARG


class TestRemoveBiosPass(TestBase):
    option_arg = "--remove-bios-password"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_rm_bios_pass_ok(self, mock_get, mock_post, mock_delete):
        responses_get = [
            RESET_TYPE_RESP,
            STATE_OFF_RESP,
            TASK_OK_RESP,
        ]
        responses = INIT_RESP + responses_get
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--old-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_RM_GOOD

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_rm_bios_pass_missing_arg(self, mock_get, mock_post, mock_delete):
        responses_get = [
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_get
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_RM_MISS_ARG


class TestChangeBiosPass(TestBase):
    option_arg = "--set-bios-password"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_set_bios_pass_not_supported(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 404], ["OK", "Not Found"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_CHANGE_NOT_SUPPORTED

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_set_bios_pass_cmd_failed(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["OK", "Bad Request"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_CHANGE_CMD_FAILED


class TestSetBiosPassCheckJobStatus(TestBase):
    option_arg = "--set-bios-password"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_sbp_check_job_bad_code(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_OFF_RESP,
            "{}",
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_SET_CHECK_JOB_STATUS_BAD_CODE

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_sbp_check_job_fail_msg(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_OFF_RESP,
            CHECK_JOB_STATUS_FAIL_MSG,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_SET_CHECK_JOB_STATUS_FAIL_MSG

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_sbp_check_job_unexpected_msg(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_OFF_RESP,
            CHECK_JOB_STATUS_UNEXPECTED_MSG_CONTENT,
            TASK_OK_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "--new-password", "new_pass"]
        _, err = self.badfish_call()
        assert err == BIOS_PASS_SET_GOOD



import os
from datetime import datetime, timedelta
from unittest.mock import patch

from tests.config import (
    BLANK_RESP,
    INIT_RESP,
    JOB_ID,
    RESPONSE_EXPORT_SCP_NO_LOCATION,
    RESPONSE_EXPORT_SCP_PASS,
    RESPONSE_EXPORT_SCP_STATUS_FAIL,
    RESPONSE_EXPORT_SCP_TIME_OUT,
    RESPONSE_GET_SCP_TARGETS_UNSUPPORTED_ERR,
    RESPONSE_GET_SCP_TARGETS_WITH_ALLOWABLES_PASS,
    RESPONSE_GET_SCP_TARGETS_WITHOUT_ALLOWABLES_ERR,
    RESPONSE_GET_SCP_TARGETS_WRONG,
    RESPONSE_IMPORT_SCP_FAIL_STATE,
    RESPONSE_IMPORT_SCP_INVALID_FILEPATH,
    RESPONSE_IMPORT_SCP_PASS,
    RESPONSE_IMPORT_SCP_STATUS_FAIL,
    RESPONSE_IMPORT_SCP_TIME_OUT,
    SCP_GET_TARGETS_ACTIONS_OEM_UNSUPPORTED,
    SCP_GET_TARGETS_ACTIONS_OEM_WITH_ALLOWABLES,
    SCP_GET_TARGETS_ACTIONS_OEM_WITHOUT_ALLOWABLES,
    SCP_MESSAGE_PERCENTAGE,
    SCP_MESSAGE_PERCENTAGE_STATE,
    STATE_ON_RESP,
)
from tests.test_base import TestBase


def fixed_datetime():
    fixed_datetime.counter = getattr(fixed_datetime, "counter", 0) + 1
    if fixed_datetime.counter < 5:
        return datetime.now()
    else:
        return datetime.now() + timedelta(minutes=15)


def export_dir_check():
    if not os.path.exists("exports"):
        os.makedirs("exports")


class TestGetSCPTargets(TestBase):
    option_arg = "--get-scp-targets"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_with_allowables_pass(self, mock_get, mock_post, mock_delete):
        responses_add = [
            SCP_GET_TARGETS_ACTIONS_OEM_WITH_ALLOWABLES,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "Export"]
        _, err = self.badfish_call()
        assert err == RESPONSE_GET_SCP_TARGETS_WITH_ALLOWABLES_PASS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_without_allowables_err(self, mock_get, mock_post, mock_delete):
        responses_add = [
            SCP_GET_TARGETS_ACTIONS_OEM_WITHOUT_ALLOWABLES,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "Export"]
        _, err = self.badfish_call()
        assert err == RESPONSE_GET_SCP_TARGETS_WITHOUT_ALLOWABLES_ERR

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_idrac_scp_unsupported_err(self, mock_get, mock_post, mock_delete):
        responses_add = [
            SCP_GET_TARGETS_ACTIONS_OEM_UNSUPPORTED,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "Export"]
        _, err = self.badfish_call()
        assert err == RESPONSE_GET_SCP_TARGETS_UNSUPPORTED_ERR

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_something_wrong(self, mock_get, mock_post, mock_delete):
        responses_add = [
            BLANK_RESP,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "Export"]
        _, err = self.badfish_call()
        assert err == RESPONSE_GET_SCP_TARGETS_WRONG


class TestExportSCP(TestBase):
    option_arg = "--export-scp"

    tests_dir = os.path.dirname(__file__)
    example_path = os.path.join(tests_dir, "fixtures/example_scp.json")

    @patch("badfish.helpers.get_now", fixed_datetime)
    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_pass(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses_get = [
            SCP_MESSAGE_PERCENTAGE % ("Ex", 15),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 30),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 45),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 60),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 75),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 90),
            SCP_MESSAGE_PERCENTAGE % ("Ex", 99),
            open(self.example_path, "r").read(),
        ]
        responses = INIT_RESP + responses_get
        headers = {"Location": f"/{JOB_ID}"}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "./exports/"]
        _, err = self.badfish_call()
        assert err == RESPONSE_EXPORT_SCP_PASS % (datetime.now().strftime("%Y-%m-%d_%H%M%S"))

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_status_fail(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses = INIT_RESP
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["OK", "Bad Request"], post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "./exports/"]
        _, err = self.badfish_call()
        assert err == RESPONSE_EXPORT_SCP_STATUS_FAIL

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_no_location(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses = INIT_RESP
        headers = {}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "./exports/"]
        _, err = self.badfish_call()
        assert err == RESPONSE_EXPORT_SCP_NO_LOCATION

    @patch("badfish.main.get_now", fixed_datetime)
    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_time_out(self, mock_get, mock_post, mock_delete):
        if hasattr(fixed_datetime, "counter"):
            setattr(fixed_datetime, "counter", 0)
        export_dir_check()
        responses = INIT_RESP + ["{}"] + ([SCP_MESSAGE_PERCENTAGE % ("Ex", 1)] * 4)
        headers = {"Location": f"/{JOB_ID}"}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "./exports/"]
        _, err = self.badfish_call()
        assert err == RESPONSE_EXPORT_SCP_TIME_OUT


class TestImportSCP(TestBase):
    option_arg = "--import-scp"

    tests_dir = os.path.dirname(__file__)
    example_path = os.path.join(tests_dir, "fixtures/example_scp.json")

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_pass(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses_get = [
            STATE_ON_RESP,
            SCP_MESSAGE_PERCENTAGE % ("Im", 15),
            SCP_MESSAGE_PERCENTAGE % ("Im", 30),
            SCP_MESSAGE_PERCENTAGE % ("Im", 45),
            SCP_MESSAGE_PERCENTAGE % ("Im", 60),
            SCP_MESSAGE_PERCENTAGE % ("Im", 75),
            SCP_MESSAGE_PERCENTAGE % ("Im", 90),
            SCP_MESSAGE_PERCENTAGE % ("Im", 99),
            SCP_MESSAGE_PERCENTAGE_STATE
            % ("Successfully imported and applied Server Configuration Profile.", 100, "Completed"),
        ]
        responses = INIT_RESP + responses_get
        headers = {"Location": f"/{JOB_ID}"}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, self.example_path]
        _, err = self.badfish_call()
        assert err == RESPONSE_IMPORT_SCP_PASS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_invalid_filepath(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, "no_file"]
        _, err = self.badfish_call()
        assert err == RESPONSE_IMPORT_SCP_INVALID_FILEPATH

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_status_fail(self, mock_get, mock_post, mock_delete):
        responses_get = [STATE_ON_RESP]
        responses = INIT_RESP + responses_get
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["OK", "Bad Request"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, self.example_path]
        _, err = self.badfish_call()
        assert err == RESPONSE_IMPORT_SCP_STATUS_FAIL

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_no_location(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses_get = [STATE_ON_RESP]
        responses = INIT_RESP + responses_get
        headers = {}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, self.example_path]
        _, err = self.badfish_call()
        assert err == RESPONSE_EXPORT_SCP_NO_LOCATION

    @patch("badfish.main.get_now", fixed_datetime)
    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_time_out(self, mock_get, mock_post, mock_delete):
        if hasattr(fixed_datetime, "counter"):
            setattr(fixed_datetime, "counter", 0)
        export_dir_check()
        responses_get = [
            STATE_ON_RESP,
            "{}",
            INIT_RESP[0],
        ] + ([SCP_MESSAGE_PERCENTAGE % ("Im", 1)] * 1000000)
        responses = INIT_RESP + responses_get
        headers = {"Location": f"/{JOB_ID}"}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, self.example_path]
        _, err = self.badfish_call()
        assert err == RESPONSE_IMPORT_SCP_TIME_OUT

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_fail_state(self, mock_get, mock_post, mock_delete):
        export_dir_check()
        responses_get = [
            STATE_ON_RESP,
            SCP_MESSAGE_PERCENTAGE % ("Im", 20),
            SCP_MESSAGE_PERCENTAGE % ("Im", 40),
            SCP_MESSAGE_PERCENTAGE % ("Im", 60),
            SCP_MESSAGE_PERCENTAGE_STATE % ("Unable to complete the Import operation.", 100, "Failed"),
        ]
        responses = INIT_RESP + responses_get
        headers = {"Location": f"/{JOB_ID}"}
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 202], ["OK", "OK"], headers=headers, post=True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, self.example_path]
        _, err = self.badfish_call()
        assert err == RESPONSE_IMPORT_SCP_FAIL_STATE



from unittest.mock import patch

from tests.config import (
    INIT_RESP,
    NO_POWER,
    POWER_CONSUMED_RESP,
    RESPONSE_NO_POWER_CONSUMED,
    RESPONSE_POWER_CONSUMED_OK,
    RESPONSE_POWER_CONSUMED_VAL_ERR,
    RESPONSE_VENDOR_UNSUPPORTED,
)
from tests.test_base import TestBase


class TestPowerConsumed(TestBase):
    args = ["--get-power-consumed"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_consumed(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [POWER_CONSUMED_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_CONSUMED_OK

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_consumed_404(self, mock_get, mock_post, mock_delete):
        # Mock the get_request method to return a 404 response that properly triggers vendor error
        with patch("badfish.main.Badfish.get_request") as mock_get_request:
            from tests.test_base import MockResponse

            # Mock get_request to return 404 for power endpoint
            mock_get_request.return_value = MockResponse('{"error": "Not Found"}', 404)

            self.set_mock_response(mock_get, 200, INIT_RESP)
            self.set_mock_response(mock_post, 200, "OK", True)
            self.set_mock_response(mock_delete, 200, "OK")
            _, err = self.badfish_call()
            assert err == f"{RESPONSE_VENDOR_UNSUPPORTED}\n"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_no_power(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [NO_POWER]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_NO_POWER_CONSUMED

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_consumed_value_error(self, mock_get, mock_post, mock_delete):
        responses_add = [""]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_CONSUMED_VAL_ERR



from unittest.mock import patch

from tests.config import (
    BOOT_SEQ_RESPONSE_DIRECTOR,
    INIT_RESP,
    JOB_OK_RESP,
    RESPONSE_POWER_OFF_ALREADY,
    RESPONSE_POWER_OFF_MISS_STATE,
    RESPONSE_POWER_OFF_NO_STATE,
    RESPONSE_POWER_OFF_NONE,
    RESPONSE_POWER_ON_NOT,
    RESPONSE_POWER_ON_OK,
    RESPONSE_POWER_STATE_DOWN,
    RESPONSE_POWER_STATE_EMPTY,
    RESPONSE_POWER_STATE_ON,
    STATE_OFF_RESP,
    STATE_ON_RESP,
)
from tests.test_base import TestBase


class TestPowerOn(TestBase):
    args = ["--power-on"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_on_ok(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [STATE_OFF_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 204], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_ON_OK

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_on_not_ok(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [STATE_OFF_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 409], ["OK", "Conflict"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_ON_NOT


class TestPowerOff(TestBase):
    args = ["--power-off"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_off_no_state(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + ["{}"]
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 400], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_OFF_NO_STATE

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_off_already(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [STATE_OFF_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 409], ["OK", "Conflict"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_OFF_ALREADY

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_miss_state(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [JOB_OK_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_OFF_MISS_STATE

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    @patch("badfish.main.Badfish.get_request")
    def test_power_off_none(self, mock_get_req_call, mock_get, mock_post, mock_delete):
        # The power off operation should return None when getting power state
        mock_get_req_call.side_effect = [None]
        self.set_mock_response(mock_get, 200, INIT_RESP)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_OFF_NONE


class TestPowerState(TestBase):
    args = ["--power-state"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_state(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [STATE_ON_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_STATE_ON

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_state_bad_request(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + ["Bad Request"]
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 400], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_STATE_DOWN

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_power_state_empty_data(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + ["{}"]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == RESPONSE_POWER_STATE_EMPTY

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    @patch("badfish.main.Badfish.get_request")
    def test_power_state_none(self, mock_get_req_call, mock_get, mock_post, mock_delete):
        # The power state check should return None (simulating communication failure)
        mock_get_req_call.side_effect = [None]
        # Add extra response for the power state call that should fail (404)
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 404], INIT_RESP + [""])
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == "- INFO     - Power state:\n- INFO     -     f01-h01-000-r630.host.io: 'Down'\n"



import unittest
from unittest.mock import patch, MagicMock
from badfish.main import main


class TestAsyncioFix(unittest.TestCase):
    @patch('badfish.main.execute_badfish')
    @patch('badfish.main.BadfishLogger')
    @patch('badfish.main.parse_arguments')
    @patch('asyncio.set_event_loop')
    @patch('asyncio.new_event_loop')
    @patch('asyncio.get_event_loop')
    def test_main_handles_no_event_loop(self, mock_get_loop, mock_new_loop,
                                        mock_set_loop, mock_parse_args,
                                        mock_logger, mock_execute):
        mock_get_loop.side_effect = RuntimeError("No event loop")

        mock_loop_instance = MagicMock()
        mock_new_loop.return_value = mock_loop_instance
        mock_loop_instance.run_until_complete.return_value = ("localhost", True)

        mock_parse_args.return_value = {
            "verbose": False, "host": "localhost", "delta": None,
            "firmware_inventory": None, "host_list": None, "log": None,
            "output": None
        }

        main()

        mock_get_loop.assert_called_once()
        mock_new_loop.assert_called_once()
        mock_set_loop.assert_called_once_with(mock_loop_instance)
        mock_loop_instance.run_until_complete.assert_called()

    @patch('badfish.main.execute_badfish')
    @patch('badfish.main.BadfishLogger')
    @patch('badfish.main.parse_arguments')
    @patch('asyncio.set_event_loop')
    @patch('asyncio.new_event_loop')
    @patch('asyncio.get_event_loop')
    def test_main_uses_existing_loop(self, mock_get_loop, mock_new_loop,
                                     mock_set_loop, mock_parse_args,
                                     mock_logger, mock_execute):
        existing_loop = MagicMock()
        mock_get_loop.return_value = existing_loop
        mock_get_loop.side_effect = None
        existing_loop.run_until_complete.return_value = ("localhost", True)

        mock_parse_args.return_value = {
            "verbose": False, "host": "localhost", "delta": None,
            "firmware_inventory": None, "host_list": None, "log": None,
            "output": None
        }

        main()

        mock_get_loop.assert_called_once()
        mock_new_loop.assert_not_called()
        mock_set_loop.assert_not_called()
        existing_loop.run_until_complete.assert_called()



import pytest
from unittest.mock import AsyncMock, patch, MagicMock
from badfish.main import Badfish, badfish_factory
from badfish.helpers.exceptions import BadfishException


class MockLogger:
    def __init__(self):
        self.debug_calls = []
        self.warning_calls = []
        self.info_calls = []
        self.error_calls = []

    def debug(self, message):
        self.debug_calls.append(message)

    def warning(self, message):
        self.warning_calls.append(message)

    def info(self, message):
        self.info_calls.append(message)

    def error(self, message):
        self.error_calls.append(message)


MOCK_HOST = "test-host.example.com"
MOCK_USERNAME = "test_user"
MOCK_PASSWORD = "test_password"
MOCK_RETRIES = 5


class TestContextManager:
    """Test cases for the async context manager methods."""

    @pytest.mark.asyncio
    async def test_context_manager_successful_entry_exit(self):
        """Test successful entry and exit of the context manager."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        # Mock the init method
        with patch.object(badfish_instance, "init", new_callable=AsyncMock) as mock_init:
            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                async with badfish_instance as bf:
                    assert bf is badfish_instance
                    mock_init.assert_called_once()

                mock_delete.assert_called_once()

    @pytest.mark.asyncio
    async def test_context_manager_exception_handling(self):
        """Test that exceptions are properly handled and logged."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        # Mock the init method to raise an exception
        with patch.object(badfish_instance, "init", new_callable=AsyncMock) as mock_init:
            mock_init.side_effect = BadfishException("Test exception")

            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                with pytest.raises(BadfishException):
                    async with badfish_instance:
                        pass

                # When init fails, __aexit__ is not called because the exception is raised before entering the context
                mock_delete.assert_not_called()

    @pytest.mark.asyncio
    async def test_context_manager_direct_method_calls(self):
        """Test direct calls to __aenter__ and __aexit__ methods."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        with patch.object(badfish_instance, "init", new_callable=AsyncMock) as mock_init:
            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                # Test direct __aenter__ call
                result = await badfish_instance.__aenter__()
                assert result is badfish_instance
                mock_init.assert_called_once()

                # Test direct __aexit__ call
                await badfish_instance.__aexit__(None, None, None)
                mock_delete.assert_called_once()

    @pytest.mark.asyncio
    async def test_context_manager_nested_usage(self):
        """Test nested usage of the context manager."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        with patch.object(badfish_instance, "init", new_callable=AsyncMock) as mock_init:
            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                # First context manager usage
                async with badfish_instance as bf1:
                    assert bf1 is badfish_instance

                    # Nested context manager usage (should call init again)
                    async with badfish_instance as bf2:
                        assert bf2 is badfish_instance

                # delete_session should be called twice (once for each context)
                assert mock_delete.call_count == 2
                # init should be called twice (once for each context entry)
                assert mock_init.call_count == 2

    @pytest.mark.asyncio
    async def test_context_manager_integration_with_factory(self):
        """Test context manager integration with badfish_factory."""
        logger = MockLogger()

        with patch.object(Badfish, "init", new_callable=AsyncMock) as mock_init:
            with patch.object(Badfish, "delete_session", new_callable=AsyncMock) as mock_delete:
                async with await badfish_factory(
                    MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES
                ) as badfish:
                    assert isinstance(badfish, Badfish)
                    assert badfish.host == MOCK_HOST
                    assert badfish.username == MOCK_USERNAME
                    assert badfish.password == MOCK_PASSWORD
                    # badfish_factory calls init, then context manager calls it again
                    assert mock_init.call_count == 2

                mock_delete.assert_called_once()

    @pytest.mark.asyncio
    async def test_context_manager_session_cleanup(self):
        """Test that session cleanup happens even with exceptions."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        with patch.object(badfish_instance, "init", new_callable=AsyncMock):
            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                try:
                    async with badfish_instance:
                        raise ValueError("Unexpected error")
                except ValueError:
                    pass

                # delete_session should still be called despite the exception
                mock_delete.assert_called_once()

    @pytest.mark.asyncio
    async def test_context_manager_init_failure(self):
        """Test context manager behavior when init fails."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        with patch.object(badfish_instance, "init", new_callable=AsyncMock) as mock_init:
            mock_init.side_effect = BadfishException("Init failed")

            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                with pytest.raises(BadfishException):
                    async with badfish_instance:
                        pass

                # When init fails, __aexit__ is not called
                mock_delete.assert_not_called()

    @pytest.mark.asyncio
    async def test_context_manager_delete_session_failure(self):
        """Test context manager behavior when delete_session fails."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)

        with patch.object(badfish_instance, "init", new_callable=AsyncMock):
            with patch.object(badfish_instance, "delete_session", new_callable=AsyncMock) as mock_delete:
                mock_delete.side_effect = BadfishException("Delete session failed")

                # The exception should be re-raised from __aexit__
                with pytest.raises(BadfishException):
                    async with badfish_instance:
                        pass


class TestDeleteSession:
    """Test cases for the delete_session method."""

    @pytest.mark.asyncio
    async def test_delete_session_success(self):
        """Test successful session deletion."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        # Mock the delete_request method
        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            # Mock response with status 200
            mock_response = MagicMock()
            mock_response.status = 200
            mock_delete_request.return_value = mock_response

            await badfish_instance.delete_session()

            # Verify delete_request was called with correct parameters
            mock_delete_request.assert_called_once()
            call_args = mock_delete_request.call_args
            assert call_args[0][0] == f"https://{MOCK_HOST}/redfish/v1/SessionService/Sessions/123"
            assert call_args[1]["headers"] == {"content-type": "application/json"}

            # Verify session_id and token are cleared
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

            # Verify success message was logged
            assert any("Session successfully deleted" in call for call in logger.debug_calls)

    @pytest.mark.asyncio
    async def test_delete_session_404_status(self):
        """Test session deletion with 404 status (session not found)."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            # Mock response with status 404
            mock_response = MagicMock()
            mock_response.status = 404
            mock_delete_request.return_value = mock_response

            await badfish_instance.delete_session()

            # Verify session_id and token are still cleared
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

            # Verify appropriate message was logged
            assert any("Session not found (404)" in call for call in logger.debug_calls)

    @pytest.mark.asyncio
    async def test_delete_session_unexpected_status(self):
        """Test session deletion with unexpected status code."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            # Mock response with unexpected status
            mock_response = MagicMock()
            mock_response.status = 500
            mock_delete_request.return_value = mock_response

            await badfish_instance.delete_session()

            # Verify session_id and token are still cleared
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

            # Verify warning was logged
            assert any("Unexpected status 500" in call for call in logger.warning_calls)

    @pytest.mark.asyncio
    async def test_delete_session_exception_handling(self):
        """Test session deletion when delete_request raises an exception."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            mock_delete_request.side_effect = BadfishException("Network error")

            await badfish_instance.delete_session()

            # Verify session_id and token are still cleared despite the exception
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

            # Verify exception was logged as warning
            assert any("Failed to delete session" in call for call in logger.warning_calls)
            assert any("Network error" in call for call in logger.warning_calls)

    @pytest.mark.asyncio
    async def test_delete_session_no_session_id(self):
        """Test delete_session when no session_id is set."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = None
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            await badfish_instance.delete_session()

            # delete_request should not be called
            mock_delete_request.assert_not_called()

            # token should still be cleared
            assert badfish_instance.token is None

            # Verify appropriate message was logged
            assert any("No session ID found" in call for call in logger.debug_calls)

    @pytest.mark.asyncio
    async def test_delete_session_cleanup_always_executes(self):
        """Test that cleanup (clearing session_id and token) always executes."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            # Mock response to raise an exception
            mock_delete_request.side_effect = Exception("Unexpected error")

            # The method should not raise the exception due to try/finally
            await badfish_instance.delete_session()

            # Verify session_id and token are still cleared despite the exception
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

    @pytest.mark.asyncio
    async def test_delete_session_other_exception(self):
        """Test delete_session with non-BadfishException."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
            # Mock response to raise a different exception
            mock_delete_request.side_effect = ValueError("Unexpected error")

            # The method should not raise the exception due to try/finally
            await badfish_instance.delete_session()

            # Verify session_id and token are still cleared
            assert badfish_instance.session_id is None
            assert badfish_instance.token is None

    @pytest.mark.asyncio
    async def test_delete_session_outer_exception_handler(self):
        """Test the outer exception handler that ensures cleanup even for unexpected exceptions."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = "/redfish/v1/SessionService/Sessions/123"
        badfish_instance.token = "test_token"

        # Mock the logger to raise an exception when debug is called
        # This will trigger the outer exception handler
        with patch.object(logger, "debug", side_effect=RuntimeError("Logger error")):
            with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
                # Mock successful response
                mock_response = MagicMock()
                mock_response.status = 200
                mock_delete_request.return_value = mock_response

                # The method should not raise the exception due to outer exception handler
                await badfish_instance.delete_session()

                # Verify session_id and token are still cleared despite the logger exception
                assert badfish_instance.session_id is None
                assert badfish_instance.token is None

                # Verify delete_request was called
                mock_delete_request.assert_called_once()

    @pytest.mark.asyncio
    async def test_delete_session_no_session_id_outer_exception(self):
        """Test outer exception handler when session_id is None and logger fails."""
        logger = MockLogger()
        badfish_instance = Badfish(MOCK_HOST, MOCK_USERNAME, MOCK_PASSWORD, logger, MOCK_RETRIES)
        badfish_instance.session_id = None
        badfish_instance.token = "test_token"

        # Mock the logger to raise an exception when debug is called
        with patch.object(logger, "debug", side_effect=RuntimeError("Logger error")):
            with patch.object(badfish_instance, "delete_request", new_callable=AsyncMock) as mock_delete_request:
                # The method should not raise the exception due to outer exception handler
                await badfish_instance.delete_session()

                # Verify session_id and token are still cleared despite the logger exception
                assert badfish_instance.session_id is None
                assert badfish_instance.token is None

                # Verify delete_request was not called (since session_id was None)
                mock_delete_request.assert_not_called()


class TestExecuteBadfishSessionCleanup:
    """Test cases for session cleanup in execute_badfish function."""

    @pytest.mark.asyncio
    async def test_execute_badfish_session_cleanup_success(self):
        """Test successful session cleanup in execute_badfish."""
        from badfish.main import execute_badfish

        logger = MockLogger()
        args = {
            "u": MOCK_USERNAME,
            "p": MOCK_PASSWORD,
            "t": None,
            "i": None,
            "host_list": None,
            "output": None,
            "force": False,
            "pxe": False,
            "boot_to": None,
            "boot_to_type": None,
            "boot_to_mac": None,
            "reboot_only": False,
            "power_state": False,
            "power_on": False,
            "power_off": False,
            "power_cycle": False,
            "get_power_consumed": False,
            "racreset": False,
            "bmc_reset": False,
            "factory_reset": False,
            "check_boot": False,
            "toggle_boot_device": None,
            "firmware_inventory": False,
            "clear_jobs": False,
            "check_job": None,
            "ls_jobs": False,
            "ls_interfaces": False,
            "ls_processors": False,
            "ls_gpu": False,
            "ls_memory": False,
            "ls_serial": False,
            "check_virtual_media": False,
            "unmount_virtual_media": False,
            "mount_virtual_media": None,
            "boot_to_virtual_media": False,
            "check_remote_image": False,
            "boot_remote_image": None,
            "detach_remote_image": False,
            "get_sriov": False,
            "enable_sriov": False,
            "disable_sriov": False,
            "set_bios_attribute": False,
            "get_bios_attribute": False,
            "attribute": "",
            "value": "",
            "set_bios_password": False,
            "remove_bios_password": False,
            "new_password": "",
            "old_password": "",
            "screenshot": False,
            "retries": MOCK_RETRIES,
            "get_scp_targets": None,
            "scp_targets": "ALL",
            "scp_include_read_only": False,
            "export_scp": None,
            "import_scp": None,
            "get_nic_fqdds": False,
            "get_nic_attribute": None,
            "set_nic_attribute": None,
        }

        with patch("badfish.main.badfish_factory") as mock_factory:
            # Mock badfish instance
            mock_badfish = MagicMock()
            mock_badfish.session_id = "/redfish/v1/SessionService/Sessions/123"
            mock_factory.return_value = mock_badfish

            # Mock successful operation
            with patch.object(mock_badfish, "delete_session", new_callable=AsyncMock) as mock_delete:
                host, result = await execute_badfish(MOCK_HOST, args, logger)

                assert host == MOCK_HOST
                assert result is True
                mock_delete.assert_called_once()
                assert any("Session closed for host" in call for call in logger.debug_calls)

    @pytest.mark.asyncio
    async def test_execute_badfish_session_cleanup_failure(self):
        """Test session cleanup failure in execute_badfish."""
        from badfish.main import execute_badfish

        logger = MockLogger()
        args = {
            "u": MOCK_USERNAME,
            "p": MOCK_PASSWORD,
            "t": None,
            "i": None,
            "host_list": None,
            "output": None,
            "force": False,
            "pxe": False,
            "boot_to": None,
            "boot_to_type": None,
            "boot_to_mac": None,
            "reboot_only": False,
            "power_state": False,
            "power_on": False,
            "power_off": False,
            "power_cycle": False,
            "get_power_consumed": False,
            "racreset": False,
            "bmc_reset": False,
            "factory_reset": False,
            "check_boot": False,
            "toggle_boot_device": None,
            "firmware_inventory": False,
            "clear_jobs": False,
            "check_job": None,
            "ls_jobs": False,
            "ls_interfaces": False,
            "ls_processors": False,
            "ls_gpu": False,
            "ls_memory": False,
            "ls_serial": False,
            "check_virtual_media": False,
            "unmount_virtual_media": False,
            "mount_virtual_media": None,
            "boot_to_virtual_media": False,
            "check_remote_image": False,
            "boot_remote_image": None,
            "detach_remote_image": False,
            "get_sriov": False,
            "enable_sriov": False,
            "disable_sriov": False,
            "set_bios_attribute": False,
            "get_bios_attribute": False,
            "attribute": "",
            "value": "",
            "set_bios_password": False,
            "remove_bios_password": False,
            "new_password": "",
            "old_password": "",
            "screenshot": False,
            "retries": MOCK_RETRIES,
            "get_scp_targets": None,
            "scp_targets": "ALL",
            "scp_include_read_only": False,
            "export_scp": None,
            "import_scp": None,
            "get_nic_fqdds": False,
            "get_nic_attribute": None,
            "set_nic_attribute": None,
        }

        with patch("badfish.main.badfish_factory") as mock_factory:
            # Mock badfish instance
            mock_badfish = MagicMock()
            mock_badfish.session_id = "/redfish/v1/SessionService/Sessions/123"
            mock_factory.return_value = mock_badfish

            # Mock delete_session to raise an exception
            with patch.object(mock_badfish, "delete_session", new_callable=AsyncMock) as mock_delete:
                mock_delete.side_effect = BadfishException("Session cleanup failed")

                host, result = await execute_badfish(MOCK_HOST, args, logger)

                assert host == MOCK_HOST
                assert result is True
                mock_delete.assert_called_once()
                # Verify warning message was logged
                assert any("Failed to close session for" in call for call in logger.warning_calls)
                assert any("Session cleanup failed" in call for call in logger.warning_calls)

    @pytest.mark.asyncio
    async def test_execute_badfish_no_session_cleanup(self):
        """Test execute_badfish when no session exists to clean up."""
        from badfish.main import execute_badfish

        logger = MockLogger()
        args = {
            "u": MOCK_USERNAME,
            "p": MOCK_PASSWORD,
            "t": None,
            "i": None,
            "host_list": None,
            "output": None,
            "force": False,
            "pxe": False,
            "boot_to": None,
            "boot_to_type": None,
            "boot_to_mac": None,
            "reboot_only": False,
            "power_state": False,
            "power_on": False,
            "power_off": False,
            "power_cycle": False,
            "get_power_consumed": False,
            "racreset": False,
            "bmc_reset": False,
            "factory_reset": False,
            "check_boot": False,
            "toggle_boot_device": None,
            "firmware_inventory": False,
            "clear_jobs": False,
            "check_job": None,
            "ls_jobs": False,
            "ls_interfaces": False,
            "ls_processors": False,
            "ls_gpu": False,
            "ls_memory": False,
            "ls_serial": False,
            "check_virtual_media": False,
            "unmount_virtual_media": False,
            "mount_virtual_media": None,
            "boot_to_virtual_media": False,
            "check_remote_image": False,
            "boot_remote_image": None,
            "detach_remote_image": False,
            "get_sriov": False,
            "enable_sriov": False,
            "disable_sriov": False,
            "set_bios_attribute": False,
            "get_bios_attribute": False,
            "attribute": "",
            "value": "",
            "set_bios_password": False,
            "remove_bios_password": False,
            "new_password": "",
            "old_password": "",
            "screenshot": False,
            "retries": MOCK_RETRIES,
            "get_scp_targets": None,
            "scp_targets": "ALL",
            "scp_include_read_only": False,
            "export_scp": None,
            "import_scp": None,
            "get_nic_fqdds": False,
            "get_nic_attribute": None,
            "set_nic_attribute": None,
        }

        with patch("badfish.main.badfish_factory") as mock_factory:
            # Mock badfish instance with no session_id
            mock_badfish = MagicMock()
            mock_badfish.session_id = None
            mock_factory.return_value = mock_badfish

            # Mock delete_session
            with patch.object(mock_badfish, "delete_session", new_callable=AsyncMock) as mock_delete:
                host, result = await execute_badfish(MOCK_HOST, args, logger)

                assert host == MOCK_HOST
                assert result is True
                # delete_session should not be called when session_id is None
                mock_delete.assert_not_called()

    @pytest.mark.asyncio
    async def test_execute_badfish_no_badfish_instance(self):
        """Test execute_badfish when badfish instance is None."""
        from badfish.main import execute_badfish

        logger = MockLogger()
        args = {
            "u": MOCK_USERNAME,
            "p": MOCK_PASSWORD,
            "t": None,
            "i": None,
            "host_list": None,
            "output": None,
            "force": False,
            "pxe": False,
            "boot_to": None,
            "boot_to_type": None,
            "boot_to_mac": None,
            "reboot_only": False,
            "power_state": False,
            "power_on": False,
            "power_off": False,
            "power_cycle": False,
            "get_power_consumed": False,
            "racreset": False,
            "bmc_reset": False,
            "factory_reset": False,
            "check_boot": False,
            "toggle_boot_device": None,
            "firmware_inventory": False,
            "clear_jobs": False,
            "check_job": None,
            "ls_jobs": False,
            "ls_interfaces": False,
            "ls_processors": False,
            "ls_gpu": False,
            "ls_memory": False,
            "ls_serial": False,
            "check_virtual_media": False,
            "unmount_virtual_media": False,
            "mount_virtual_media": None,
            "boot_to_virtual_media": False,
            "check_remote_image": False,
            "boot_remote_image": None,
            "detach_remote_image": False,
            "get_sriov": False,
            "enable_sriov": False,
            "disable_sriov": False,
            "set_bios_attribute": False,
            "get_bios_attribute": False,
            "attribute": "",
            "value": "",
            "set_bios_password": False,
            "remove_bios_password": False,
            "new_password": "",
            "old_password": "",
            "screenshot": False,
            "retries": MOCK_RETRIES,
            "get_scp_targets": None,
            "scp_targets": "ALL",
            "scp_include_read_only": False,
            "export_scp": None,
            "import_scp": None,
            "get_nic_fqdds": False,
            "get_nic_attribute": None,
            "set_nic_attribute": None,
        }

        with patch("badfish.main.badfish_factory") as mock_factory:
            # Mock badfish_factory to raise an exception
            mock_factory.side_effect = BadfishException("Connection failed")

            host, result = await execute_badfish(MOCK_HOST, args, logger)

            assert host == MOCK_HOST
            assert result is False
            # No session cleanup should happen when badfish is None
            assert not any("Session closed for host" in call for call in logger.debug_calls)
            assert not any("Failed to close session for" in call for call in logger.warning_calls)



from unittest.mock import patch

from tests.config import (
    BOOT_SEQ_RESPONSE_DIRECTOR,
    INIT_RESP,
    RESET_TYPE_NG_RESP,
    RESET_TYPE_RESP,
    RESPONSE_REBOOT_ONLY_FAILED_SEND_RESET,
    RESPONSE_REBOOT_ONLY_SUCCESS,
    RESPONSE_REBOOT_ONLY_SUCCESS_WITH_NG_RT,
    STATE_DOWN_RESP,
    STATE_OFF_RESP,
    STATE_ON_RESP,
)
from tests.test_base import TestBase


class TestRebootOnly(TestBase):
    option_arg = "--reboot-only"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reboot_only_success(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_OFF_RESP,
            STATE_ON_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 204, 204], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_REBOOT_ONLY_SUCCESS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reboot_only_success_with_polling_down(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_DOWN_RESP,
            STATE_ON_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 204, 204], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_REBOOT_ONLY_SUCCESS

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reboot_only_failed_send_reset(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_DOWN_RESP,
            STATE_ON_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["OK", "Bad Request"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_REBOOT_ONLY_FAILED_SEND_RESET

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reboot_only_success_with_ng_rt(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_NG_RESP,
            STATE_ON_RESP,
            STATE_DOWN_RESP,
            STATE_ON_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 204, 204], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_REBOOT_ONLY_SUCCESS_WITH_NG_RT

    @patch("badfish.main.RETRIES", 0)
    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.get")
    @patch("aiohttp.ClientSession.post")
    def test_reboot_only_failed_grace_and_force(self, mock_post, mock_get, mock_delete):
        responses = INIT_RESP + [
            RESET_TYPE_RESP,
            STATE_ON_RESP,
        ]
        self.set_mock_response(mock_get, 200, responses)
        # Provide enough POST responses to handle the entire reboot sequence
        self.set_mock_response(mock_post, [200] + [409] * 10, ["OK"] + ["Conflict"] * 10, True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        # The test should complete without StopIteration and show reboot failures
        # Checking for key failure indicators rather than exact match
        assert "Command failed to GracefulRestart" in err
        assert "Command failed to ForceOff" in err



from unittest.mock import patch

from tests.config import (
    BOOT_MODE_RESP,
    BOOT_SEQ_RESP,
    BOOT_SEQ_RESPONSE_DIRECTOR,
    BOOT_SEQ_RESPONSE_FOREMAN,
    BOOT_SEQ_RESPONSE_NO_MATCH,
    INIT_RESP,
    INTERFACES_PATH,
    RESPONSE_DIRECTOR,
    RESPONSE_FOREMAN,
    RESPONSE_WITHOUT,
    WARN_NO_MATCH,
)
from tests.test_base import TestBase


class TestCheckBoot(TestBase):
    option_arg = "--check-boot"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_boot_without_interfaces(self, mock_get, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        responses_add = [BOOT_MODE_RESP, boot_seq_resp_fmt.replace("'", '"')]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_WITHOUT

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_boot_with_interfaces_director(self, mock_get, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        responses_add = [BOOT_MODE_RESP, boot_seq_resp_fmt.replace("'", '"')]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = ["-i", INTERFACES_PATH, self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_DIRECTOR

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_boot_with_interfaces_foreman(self, mock_get, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_FOREMAN)
        responses_add = [BOOT_MODE_RESP, boot_seq_resp_fmt.replace("'", '"')]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = ["-i", INTERFACES_PATH, self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_FOREMAN

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_check_boot_no_match(self, mock_get, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_NO_MATCH)
        responses_add = [BOOT_MODE_RESP, boot_seq_resp_fmt.replace("'", '"')]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = ["-i", INTERFACES_PATH, self.option_arg]
        _, err = self.badfish_call()
        assert err == WARN_NO_MATCH



import json
import sys

import yaml

try:
    # Python 3.7 and newer, fast reentrant implementation
    # without task tracking (not needed for that when logging)
    from queue import SimpleQueue as Queue
except ImportError:
    from queue import Queue
from logging.handlers import QueueHandler, QueueListener

from logging import (
    Formatter,
    FileHandler,
    DEBUG,
    INFO,
    ERROR,
    StreamHandler,
    getLogger,
)


class NoAliasDumper(yaml.SafeDumper):
    def ignore_aliases(self, data):
        return True


class BadfishHandler(StreamHandler):
    def __init__(self, format_flag=False):
        StreamHandler.__init__(self)
        self.messages = {}
        self.formatted_msg = []
        self.output_dict = dict()
        self.host = None
        self.format_flag = format_flag

    def emit(self, record):
        if not self.format_flag:
            self.formatted_msg.append(self.formatter.format(record))
            return

        if record.levelno == INFO and record.msg != "*" * 48:
            if record.name not in self.messages:
                self.messages.update({record.name: record.msg + "\n"})
            else:
                self.messages[record.name] += record.msg + "\n"
        elif record.levelno == ERROR:
            self.output_dict = {"error": True, "error_msg": record.msg}

    def parse(self):
        try:
            if self.host:
                host_name = self.host.strip().split(".")[0]
                # Ensure the message is properly formatted as YAML by wrapping values in quotes
                message = self.messages[host_name]
                # Try to parse as is first
                try:
                    new_dict = yaml.safe_load(message)
                except yaml.YAMLError:
                    # If parsing fails, try to format the value as a quoted string
                    lines = message.strip().split("\n")
                    formatted_lines = []
                    for line in lines:
                        if ":" in line:
                            key, value = line.split(":", 1)
                            value = value.strip()
                            # If value contains spaces or special characters, wrap in quotes
                            if " " in value or any(c in value for c in "{}[](),:#"):
                                value = f'"{value}"'
                            formatted_lines.append(f"{key}: {value}")
                        else:
                            formatted_lines.append(line)
                    formatted_message = "\n".join(formatted_lines)
                    new_dict = yaml.safe_load(formatted_message)

                self.output_dict.update({self.host: new_dict.copy()})
                self.host = None
            else:
                message = self.messages["badfish.helpers.logger"]
                # Apply the same formatting logic for non-host messages
                try:
                    new_dict = yaml.safe_load(message)
                except yaml.YAMLError:
                    lines = message.strip().split("\n")
                    formatted_lines = []
                    for line in lines:
                        if ":" in line:
                            key, value = line.split(":", 1)
                            value = value.strip()
                            if " " in value or any(c in value for c in "{}[](),:#"):
                                value = f'"{value}"'
                            formatted_lines.append(f"{key}: {value}")
                        else:
                            formatted_lines.append(line)
                    formatted_message = "\n".join(formatted_lines)
                    new_dict = yaml.safe_load(formatted_message)

                self.output_dict.update(new_dict.copy())
        except yaml.YAMLError:
            self.output_dict = {"unsupported_command": True}

    def diff(self):
        try:
            if self.output_dict["error"]:
                return f"ERROR - {self.output_dict['error_msg']}"
        except KeyError:
            host_first, host_second, *_ = self.output_dict.keys()
            first, second, *_ = self.output_dict.values()
            diff_dict = {host_first: {}, host_second: {}}
            for i in first:
                for j in second:
                    if (
                        first[i]["SoftwareId"] == second[j]["SoftwareId"]
                        and first[i]["Version"] != second[j]["Version"]
                        and first[i]["SoftwareId"] != 0
                    ):
                        diff_dict[host_first].update(
                            {
                                i: {
                                    "Version": first[i]["Version"],
                                    "Name": first[i]["Name"],
                                }
                            }
                        )
                        diff_dict[host_second].update(
                            {
                                j: {
                                    "Version": second[j]["Version"],
                                    "Name": second[j]["Name"],
                                }
                            }
                        )

            if diff_dict[host_first] == {}:
                return "{}"
            output = ""
            formatted = json.dumps(diff_dict[host_first], indent=4, sort_keys=False, default=str)
            len_first = (max(len(line) for line in formatted.splitlines())) + 10
            output += f"{host_first}:".ljust(len_first)
            output += f"{host_second}:\n"
            for i, j in zip(diff_dict[host_first], diff_dict[host_second]):
                output += f"{i}".ljust(len_first)
                output += f"{j}\n"
                output += f"\t- Name: {(diff_dict[host_first][i])['Name']}".ljust(len_first)
                output += f"\t- Name: {(diff_dict[host_second][j])['Name']}\n"
                output += f"\t- Version: {(diff_dict[host_first][i])['Version']}".ljust(len_first)
                output += f"\t- Version: {(diff_dict[host_second][j])['Version']}\n"
            return output

    def output(self, output_type, host_order=None):
        if output_type == "json":
            return json.dumps(self.output_dict, indent=4, sort_keys=False, default=str)
        elif output_type == "yaml":
            return yaml.dump(
                self.output_dict,
                sort_keys=False,
                indent=4,
                default_flow_style=False,
                Dumper=NoAliasDumper,
            )
        else:
            if len(host_order) > 1:
                logger_handle = self.formatted_msg[-1].split()[0].strip("[]")
                host_order.update({logger_handle: sys.maxsize})
                sorted_msg = [
                    x
                    for x in sorted(
                        self.formatted_msg,
                        key=lambda y: host_order[y.split()[0].strip("[]")],
                    )
                ]
            else:
                sorted_msg = self.formatted_msg
            return "\n".join(sorted_msg)


class BadfishLogger:
    def __init__(self, verbose=False, multi_host=False, log_file=None, output=None):
        self.log_level = DEBUG if verbose else INFO
        self.multi_host = multi_host
        self.log_file = log_file

        _host_name_tag = "[%(name)s] " if self.multi_host else ""
        _format_str = f"{_host_name_tag}- %(levelname)-8s - %(message)s"
        _file_format_str = f"%(asctime)-12s: {_host_name_tag}- %(levelname)-8s - %(message)s"

        self.badfish_handler = BadfishHandler(True if output else False)
        self.badfish_handler.setFormatter(Formatter(_format_str))
        self.badfish_handler.setLevel(INFO)

        _queue = Queue()
        self.queue_listener = QueueListener(_queue, self.badfish_handler)
        self.queue_handler = QueueHandler(_queue)

        self.logger = getLogger(__name__)
        self.logger.addHandler(self.queue_handler)
        self.logger.setLevel(self.log_level)

        self.queue_listener.start()

        if self.log_file:
            self.file_handler = FileHandler(self.log_file)
            self.file_handler.setFormatter(Formatter(_file_format_str))
            self.file_handler.setLevel(self.log_level)
            self.queue_listener.handlers = self.queue_listener.handlers + (self.file_handler,)



from unittest.mock import patch

from tests.config import (
    BLANK_RESP,
    BOOT_MODE_RESP,
    BOOT_SEQ_RESP,
    BOOT_SEQ_RESPONSE_DIRECTOR,
    ETHERNET_INTERFACES_RESP,
    INIT_RESP,
    INTERFACES_PATH,
    INTERFACES_RESP,
    JOB_OK_RESP,
    MAC_ADDRESS,
    RESPONSE_BOOT_TO,
    RESPONSE_BOOT_TO_BAD_MAC,
)
from tests.test_base import TestBase


class TestBootToMac(TestBase):
    option_arg = "--boot-to-mac"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_good_mac(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            ETHERNET_INTERFACES_RESP,
            INTERFACES_RESP,
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = ["-i", INTERFACES_PATH, self.option_arg, MAC_ADDRESS]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_bad_mac(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            ETHERNET_INTERFACES_RESP,
            INTERFACES_RESP,
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = ["-i", INTERFACES_PATH, self.option_arg, "no:tg:oo:dm:ac"]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO_BAD_MAC



from unittest.mock import patch

from tests.config import (
    BAD_DEVICE_NAME,
    BLANK_RESP,
    BOOT_MODE_RESP,
    BOOT_SEQ_RESP,
    BOOT_SEQ_RESPONSE_DIRECTOR,
    DEVICE_NIC_2,
    ERROR_DEV_NO_MATCH,
    INIT_RESP,
    JOB_OK_RESP,
    RESET_TYPE_RESP,
    RESPONSE_BOOT_TO,
    RESPONSE_BOOT_TO_SERVICE_BAD_REQUEST,
    RESPONSE_BOOT_TO_SERVICE_ERR_HANDLER,
    RESPONSE_BOOT_TO_SERVICE_UNAVAILABLE,
    STATE_ON_RESP,
)
from tests.test_base import TestBase


class TestBootTo(TestBase):
    option_arg = "--boot-to"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, ["OK", JOB_OK_RESP])
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, DEVICE_NIC_2["name"]]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_service_unavailable(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
        ]
        responses = INIT_RESP + get_resp
        post_responses = ["OK"] + [JOB_OK_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, [503, 200], ["Service Unavailable", "OK"])
        self.set_mock_response(mock_post, 200, post_responses)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, DEVICE_NIC_2["name"]]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO_SERVICE_UNAVAILABLE

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_bad_request(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
            BLANK_RESP,
            RESET_TYPE_RESP,
            STATE_ON_RESP,
        ]
        responses = INIT_RESP + get_resp
        post_responses = ["OK"] + [JOB_OK_RESP, JOB_OK_RESP, JOB_OK_RESP, JOB_OK_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, [400, 200], ["Bad Request", "OK"])
        self.set_mock_response(mock_post, [200, 204, 400], post_responses, True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, DEVICE_NIC_2["name"]]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO_SERVICE_BAD_REQUEST

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_service_err_handler(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
            BOOT_MODE_RESP,
        ]
        responses = INIT_RESP + get_resp
        post_responses = ["OK"] + [JOB_OK_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 403, "Forbidden")
        self.set_mock_response(mock_post, 200, post_responses)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, DEVICE_NIC_2["name"]]
        _, err = self.badfish_call()
        assert err == RESPONSE_BOOT_TO_SERVICE_ERR_HANDLER

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_boot_to_no_match(self, mock_get, mock_patch, mock_post, mock_delete):
        boot_seq_resp_fmt = BOOT_SEQ_RESP % str(BOOT_SEQ_RESPONSE_DIRECTOR)
        get_resp = [
            BOOT_MODE_RESP,
            boot_seq_resp_fmt.replace("'", '"'),
            BLANK_RESP,
        ]
        responses = INIT_RESP + get_resp
        post_responses = ["OK"] + [JOB_OK_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, post_responses)
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg, BAD_DEVICE_NAME]
        _, err = self.badfish_call()
        assert err == ERROR_DEV_NO_MATCH



import pytest

from badfish.main import Badfish
from tests.config import INTERFACES_PATH
from tests.test_base import TestBase

E26_EXPECTED_INTERFACES = {
    "director": "NIC.Slot.3-1-1,HardDisk.List.1-1,NIC.Integrated.1-1-1",
    "foreman": "NIC.Integrated.1-1-1,HardDisk.List.1-1,NIC.Slot.3-1-1",
    "uefi": "NIC.Slot.3-1-1,HardDisk.List.1-1,NIC.Integrated.1-1-1",
}

E27_EXPECTED_INTERFACES = {
    "director": "NIC.Slot.3-1-1,NIC.Slot.3-2-1,NIC.Slot.6-1-1,NIC.Slot.6-2-1,NIC.Embedded.1-1-1,HardDisk.List.1-1",
    "foreman": "NIC.Embedded.1-1-1,NIC.Slot.3-1-1,NIC.Slot.3-2-1,NIC.Slot.6-1-1,NIC.Slot.6-2-1,HardDisk.List.1-1",
    "uefi": "NIC.Slot.3-1-1,HardDisk.List.1-1,NIC.Embedded.1-1-1",
}

FC640_B01_INTERFACES = {
    "uefi": "NIC.ChassisSlot.8-1-1,HardDisk.List.1-1,NIC.Integrated.1-1-1",
}

FC640_B02_INTERFACES = {
    "uefi": "NIC.ChassisSlot.4-1-1,HardDisk.List.1-1,NIC.Integrated.1-1-1",
}


class TestGetInterfaceByType(TestBase):
    @pytest.mark.asyncio
    async def test_get_interface_by_type(self):
        for host, expected in [
            ("mgmt-e26-h01-r750", E26_EXPECTED_INTERFACES),
            ("mgmt-e27-h01-r750", E27_EXPECTED_INTERFACES),
            ("mgmt-e26-h01-000-r750", E26_EXPECTED_INTERFACES),
            ("mgmt-e27-h01-000-r750", E27_EXPECTED_INTERFACES),
            ("mgmt-e99-h01-b01-fc640", FC640_B01_INTERFACES),
            ("mgmt-e99-h01-b02-fc640", FC640_B02_INTERFACES),
        ]:
            for host_type in expected:
                expected_interfaces = expected[host_type].split(",")
                with self.subTest(host=host, host_type=host_type):
                    badfish = Badfish(
                        _host=host,
                        _username="",
                        _password="",
                        _logger="",
                        _retries="",
                    )
                    interfaces = await badfish.get_interfaces_by_type(host_type, _interfaces_path=INTERFACES_PATH)
                    assert interfaces == expected_interfaces, (
                        f"{host_type} interfaces for host: {host} : "
                        f"{interfaces} does not match expected: {expected_interfaces}"
                    )



from unittest.mock import patch

from tests.config import (
    BIOS_RESPONSE_SRIOV,
    BIOS_SET_OK,
    INIT_RESP,
    JOB_OK_RESP,
    RESET_TYPE_RESP,
    SRIOV_ALREADY,
    SRIOV_STATE,
    STATE_ON_RESP,
)
from tests.test_base import TestBase


class TestSriovModeEnable(TestBase):
    args = ["--enable-sriov"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_enable_sriov_ok(self, mock_get, mock_patch, mock_post, mock_delete):
        get_resp = [
            BIOS_RESPONSE_SRIOV % "Disabled",
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_ON_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == BIOS_SET_OK

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_enable_sriov_already(self, mock_get, mock_patch, mock_post, mock_delete):
        get_resp = [
            BIOS_RESPONSE_SRIOV % "Enabled",
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_ON_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == SRIOV_ALREADY


class TestSriovModeDisable(TestBase):
    args = ["--disable-sriov"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_disable_sriov_ok(self, mock_get, mock_patch, mock_post, mock_delete):
        get_resp = [
            BIOS_RESPONSE_SRIOV % "Enabled",
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_ON_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == BIOS_SET_OK

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.patch")
    @patch("aiohttp.ClientSession.get")
    def test_disable_sriov_already(self, mock_get, mock_patch, mock_post, mock_delete):
        get_resp = [
            BIOS_RESPONSE_SRIOV % "Disabled",
            RESET_TYPE_RESP,
            STATE_ON_RESP,
            STATE_ON_RESP,
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_patch, 200, ["OK"])
        self.set_mock_response(mock_post, 200, JOB_OK_RESP)
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == SRIOV_ALREADY


class TestSriovModeGet(TestBase):
    args = ["--get-sriov"]

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_get_sriov_ok(self, mock_get, mock_post, mock_delete):
        get_resp = [
            BIOS_RESPONSE_SRIOV % "Enabled",
        ]
        responses = INIT_RESP + get_resp
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        _, err = self.badfish_call()
        assert err == SRIOV_STATE



import setuptools
import re
import os

here = os.path.abspath(os.path.dirname(__file__))
version_file = os.path.join(here, "src", "badfish", "__init__.py")

with open(version_file, "r", encoding="utf-8") as f:
    content = f.read()
    match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", content, re.M)
    if match:
        current_version = match.group(1)
    else:
        raise RuntimeError("Unable to find version string in src/badfish/__init__.py")

setuptools.setup(
    version=current_version
)



from unittest.mock import patch

from tests.config import (
    BOOT_SEQ_RESPONSE_DIRECTOR,
    INIT_RESP,
    INIT_RESP_SUPERMICRO,
    RESET_TYPE_RESP,
    RESET_TYPE_RESP_NO_ALLOWABLE_VALUES,
    RESPONSE_RESET,
    RESPONSE_RESET_FAIL,
    RESPONSE_RESET_WRONG_VENDOR,
)
from tests.test_base import TestBase


class TestResetBMC(TestBase):
    option_arg = "--bmc-reset"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reset_bmc(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP_SUPERMICRO + [RESET_TYPE_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 200], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_RESET % ("200", "BMC", "BMC")

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reset_bmc_no_allowable_values(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP_SUPERMICRO + [RESET_TYPE_RESP_NO_ALLOWABLE_VALUES]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 200], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_RESET % ("200", "BMC", "BMC")

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reset_bmc_fail(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP_SUPERMICRO + [RESET_TYPE_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 400], ["OK", "Bad Request"], True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_RESET_FAIL

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_reset_bmc_wrong_vendor(self, mock_get, mock_post, mock_delete):
        responses = INIT_RESP + [RESET_TYPE_RESP]
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, [200, 204], "OK", True)
        self.set_mock_response(mock_delete, 200, "OK")
        self.boot_seq = BOOT_SEQ_RESPONSE_DIRECTOR
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_RESET_WRONG_VENDOR % ("Supermicro", "Dell", "--racreset")



import argparse

from badfish.config import RETRIES


def create_parser():
    parser = argparse.ArgumentParser(
        prog="badfish",
        description="Tool for managing server hardware via the Redfish API.",
        allow_abbrev=False,
    )
    parser.add_argument("-H", "--host", help="iDRAC host address")
    parser.add_argument("-u", help="iDRAC username", required=True)
    parser.add_argument("-p", help="iDRAC password", required=True)
    parser.add_argument("-i", help="Path to iDRAC interfaces yaml", default=None)
    parser.add_argument("-t", help="Type of host as defined on iDRAC interfaces yaml")
    parser.add_argument("-l", "--log", help="Optional argument for logging results to a file")
    parser.add_argument(
        "-o",
        "--output",
        choices=["json", "yaml"],
        help="Optional argument for choosing a special output format (json/yaml), otherwise our normal format is used.",
    )
    parser.add_argument(
        "-f",
        "--force",
        dest="force",
        action="store_true",
        help="Optional argument for forced clear-jobs",
    )
    parser.add_argument(
        "--host-list",
        help="Path to a plain text file with a list of hosts",
        default=None,
    )
    parser.add_argument("--pxe", help="Set next boot to one-shot boot PXE", action="store_true")
    parser.add_argument("--boot-to", help="Set next boot to one-shot boot to a specific device")
    parser.add_argument(
        "--boot-to-type",
        help="Set next boot to one-shot boot to a specific type as defined on iDRAC interfaces yaml",
    )
    parser.add_argument(
        "--boot-to-mac",
        help="Set next boot to one-shot boot to a specific MAC address on the target",
    )
    parser.add_argument("--reboot-only", help="Flag for only rebooting the host", action="store_true")
    parser.add_argument(
        "--power-cycle",
        help="Flag for sending ForceOff instruction to the host",
        action="store_true",
    )
    parser.add_argument(
        "--power-state",
        help="Get power state",
        action="store_true",
    )
    parser.add_argument(
        "--power-on",
        help="Power on host",
        action="store_true",
    )
    parser.add_argument(
        "--power-off",
        help="Power off host",
        action="store_true",
    )
    parser.add_argument(
        "--get-power-consumed",
        help="Get current consumed watts on host(s)",
        action="store_true",
    )
    parser.add_argument("--racreset", help="Flag for iDRAC reset", action="store_true")
    parser.add_argument("--bmc-reset", help="Flag for BMC reset", action="store_true")
    parser.add_argument(
        "--factory-reset",
        help="Reset BIOS to default factory settings",
        action="store_true",
    )
    parser.add_argument(
        "--check-boot",
        help="Flag for checking the host boot order",
        action="store_true",
    )
    parser.add_argument(
        "--toggle-boot-device",
        help="Change the enabled status of a boot device",
        default="",
    )
    parser.add_argument(
        "--firmware-inventory",
        help="Get firmware inventory",
        action="store_true",
    )
    parser.add_argument(
        "--delta",
        help="Address of the other host between which the delta should be made",
        default="",
    )
    parser.add_argument(
        "--clear-jobs",
        help="Clear any scheduled jobs from the queue",
        action="store_true",
    )
    parser.add_argument(
        "--check-job",
        help="Check a job status and details",
    )
    parser.add_argument(
        "--ls-jobs",
        help="List any scheduled jobs in queue",
        action="store_true",
    )
    parser.add_argument(
        "--ls-interfaces",
        help="List Network interfaces",
        action="store_true",
    )
    parser.add_argument(
        "--ls-processors",
        help="List Processor Summary",
        action="store_true",
    )
    parser.add_argument(
        "--ls-gpu",
        help="List GPU's on host",
        action="store_true",
    )
    parser.add_argument(
        "--ls-memory",
        help="List Memory Summary",
        action="store_true",
    )
    parser.add_argument(
        "--ls-serial",
        help="List 'Serial Number'/'Service Tag'",
        action="store_true",
    )
    parser.add_argument(
        "--check-virtual-media",
        help="Check for mounted iso images",
        action="store_true",
    )
    parser.add_argument(
        "--mount-virtual-media",
        help="Mount iso image to virtual CD. Arguments should be the address/path to the iso.",
        default="",
    )
    parser.add_argument(
        "--unmount-virtual-media",
        help="Unmount any mounted iso images",
        action="store_true",
    )
    parser.add_argument(
        "--boot-to-virtual-media",
        help="Boot to virtual media (Cd).",
        action="store_true",
    )
    parser.add_argument(
        "--check-remote-image",
        help="Check the attach status of network ISO.",
        action="store_true",
    )
    parser.add_argument(
        "--boot-remote-image",
        help="Boot to network ISO, through NFS, takes two arguments 'hostname:path' and name of the ISO 'linux.iso'.",
        default="",
    )
    parser.add_argument(
        "--detach-remote-image",
        help="Remove attached network ISO.",
        action="store_true",
    )
    parser.add_argument(
        "--get-sriov",
        help="Gets global SRIOV mode state",
        action="store_true",
    )
    parser.add_argument(
        "--enable-sriov",
        help="Enables global SRIOV mode",
        action="store_true",
    )
    parser.add_argument(
        "--disable-sriov",
        help="Disables global SRIOV mode",
        action="store_true",
    )
    parser.add_argument(
        "--get-bios-attribute",
        help="Get a BIOS attribute value",
        action="store_true",
    )
    parser.add_argument(
        "--set-bios-attribute",
        help="Set a BIOS attribute value",
        action="store_true",
    )
    parser.add_argument(
        "--attribute",
        help="BIOS attribute name",
        default="",
    )
    parser.add_argument(
        "--value",
        help="BIOS attribute value",
        default="",
    )
    parser.add_argument(
        "--set-bios-password",
        help="Set the BIOS password",
        action="store_true",
    )
    parser.add_argument(
        "--remove-bios-password",
        help="Removes BIOS password",
        action="store_true",
    )
    parser.add_argument(
        "--new-password",
        help="The new password value",
        default="",
    )
    parser.add_argument(
        "--old-password",
        help="The old password value",
        default="",
    )
    parser.add_argument(
        "--screenshot",
        help="Take a screenshot of the system an store it in jpg format",
        action="store_true",
    )
    parser.add_argument("-v", "--verbose", help="Verbose output", action="store_true")
    parser.add_argument(
        "-r",
        "--retries",
        help="Number of retries for executing actions.",
        default=RETRIES,
    )
    parser.add_argument(
        "--get-scp-targets",
        help="Get allowable target values to export or import with iDRAC SCP. Choices=['Export', 'Import']",
        choices=["Export", "Import"],
        default="",
    )
    parser.add_argument(
        "--scp-targets",
        help="Comma separated targets which configs should be exported with iDRAC SCP.",
        default="ALL",
    )
    parser.add_argument(
        "--scp-include-read-only",
        help="Flag for including read only attributes in SCP export.",
        action="store_true",
    )
    parser.add_argument(
        "--export-scp",
        help="Export system config using iDRAC SCP, argument specifies where file should be saved.",
        default="",
    )
    parser.add_argument(
        "--import-scp",
        help="Import system config using iDRAC SCP, argument specifies which JSON file contains config that should be "
        "imported.",
        default="",
    )
    parser.add_argument(
        "--get-nic-fqdds",
        help="List FQDDs for all NICs.",
        action="store_true",
    )
    parser.add_argument(
        "--get-nic-attribute",
        help="Get a NIC attribute values, specify a NIC FQDD.",
        default="",
    )
    parser.add_argument(
        "--set-nic-attribute",
        help="Set a NIC attribute value",
        default="",
    )
    return parser


def parse_arguments(argv=None):
    """Parse command line arguments using the configured parser."""
    parser = create_parser()
    return vars(parser.parse_args(argv))



from unittest.mock import patch

from tests.config import (
    GPU_DATA_RESP1,
    GPU_DATA_RESP2,
    GPU_DATA_RESP_FAULTY,
    GPU_MEMBERS_RESP,
    GPU_SUMMARY_RESP_FAULTY,
    INIT_RESP,
    MOCK_HOST,
    RESPONSE_LS_GPU,
    RESPONSE_LS_GPU_SUMMARY_DATA_ERROR,
    RESPONSE_LS_GPU_SUMMARY_VALUE_ERROR,
)
from tests.test_base import TestBase


class TestLsGpu(TestBase):
    option_arg = "--ls-gpu"

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_gpu(self, mock_get, mock_post, mock_delete):
        responses_add = [GPU_MEMBERS_RESP, GPU_DATA_RESP1, GPU_DATA_RESP2]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_delete, 200, "OK")
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_get, 200, responses)
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_LS_GPU

    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_gpu_data_not_available(self, mock_get, mock_post):
        responses_add = [
            GPU_SUMMARY_RESP_FAULTY,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 404], responses)
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_LS_GPU_SUMMARY_DATA_ERROR

    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_gpu_summary_data_error(self, mock_get, mock_post):
        responses_add = [
            GPU_MEMBERS_RESP,
            GPU_DATA_RESP1,
            GPU_DATA_RESP_FAULTY,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, [200, 200, 200, 200, 200, 404, 200, 200], responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert (
            err
            == RESPONSE_LS_GPU_SUMMARY_VALUE_ERROR
            + f"- WARNING  - Failed to delete session for {MOCK_HOST}: Failed to communicate with server.\n"
        )

    @patch("aiohttp.ClientSession.delete")
    @patch("aiohttp.ClientSession.post")
    @patch("aiohttp.ClientSession.get")
    def test_ls_gpu_summary_value_error(self, mock_get, mock_post, mock_delete):
        responses_add = [
            GPU_MEMBERS_RESP,
            GPU_DATA_RESP1,
            GPU_DATA_RESP_FAULTY,
        ]
        responses = INIT_RESP + responses_add
        self.set_mock_response(mock_get, 200, responses)
        self.set_mock_response(mock_post, 200, "OK")
        self.set_mock_response(mock_delete, 200, "OK")
        self.args = [self.option_arg]
        _, err = self.badfish_call()
        assert err == RESPONSE_LS_GPU_SUMMARY_VALUE_ERROR
